unit MenuManagerUnit;

interface

uses
  inifiles, strutils, menus, extctrls,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BPLBase, BPLClient, ImgList, ComCtrls;

type
  TForm1 = class(TForm)
    BPLMenuManager: TDynBPLClient;
    procedure FormCreate(Sender: TObject);
    procedure BPLMenuManagerAppObjectNotification(Sender: TObject;
      AppObject: TAppObject; Notification: TAppObjectNotification);
    procedure BPLMenuManagerServiceCall(Sender: TObject;
      ServiceText: String; ServiceNumber: Integer; Subject: TObject;
      out handled: Boolean);
    procedure FormDestroy(Sender: TObject);
  private
    _Closing: boolean;
    MainMenu: TMainMenu;

    procedure AddMenuItem(AppObject: TAppObject);
    procedure OnItemClick(Sender: TObject);
    function GetOrCreateSubMenuItem(Path: string): TMenuItem;
    function ValidName(Name: string): string;
    function FindSubMenuItem(SubMenu: TMenuItem; Name: string): TMenuItem;
    function FindSubMenu(Name: string): TMenuItem;

    procedure RemoveEmptyLinks(FromItem: TMenuItem; AppObject: TAppObject);
    procedure RemoveDataLinks(FromItem: TMenuItem; AppObject: TAppObject);
  public
    { Public declarations }
    procedure OnMenuClick(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation


{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  _Closing := false;
  MainMenu := TMainMenu.Create(Application.MainForm);
  BPLMenuManager.IterateAppObjects;
end;

procedure TForm1.BPLMenuManagerAppObjectNotification(Sender: TObject;
  AppObject: TAppObject; Notification: TAppObjectNotification);
begin
  if _Closing then
    exit;

  case Notification of
    noRegister,
    noIterate:
      begin
        case AppObject.ObjectType of
          otConfig,
          otCommand:
            begin
              AddMenuItem(AppObject);
            end;
        end;
      end;
    noUnregister,
    noComponentUnassigning:
      begin
        case AppObject.ObjectType of
          otConfig,
          otCommand:
            begin
              RemoveDataLinks(MainMenu.Items, AppObject);
              RemoveEmptyLinks(MainMenu.Items, AppObject);
            end;
        end;
      end;
  end;

end;



procedure TForm1.OnItemClick(Sender: TObject);
var
  obj: TAppObject;
begin
  obj := TAppObject( TMenuItem(Sender).Tag );
  if obj <> nil then
    obj.Execute;
end;

procedure TForm1.BPLMenuManagerServiceCall(Sender: TObject;
  ServiceText: String; ServiceNumber: Integer; Subject: TObject;
  out handled: Boolean);
begin
  if ServiceNumber = 1 {App.End} then
    _Closing := true;
end;


function TForm1.GetOrCreateSubMenuItem(Path: string): TMenuItem;
var
  i: integer;
  sl: TStringList;
  menuItem: TMenuItem;
  name: string;
  subMenu: TMenuItem;
begin
  Result := nil;
  menuItem := nil;
  subMenu := MainMenu.Items;

  sl := TStringList.Create;
  try
    sl.Delimiter := '.';
    sl.DelimitedText := Path;

    name := ValidName('SubMenuItem_' + Path);
    Result := FindSubMenu(name);
    if Result <> nil then
      exit;

    Name := 'SubMenuItem';
    for i:=0 to sl.Count-1 do
    begin
      Name := ValidName( Name + '_' + sl.Strings[i]);

      menuItem := FindSubMenu(Name);
      if menuItem <> nil then
        exit;

      menuItem := TMenuItem.Create(self);
      menuItem.Name := Name;
      menuItem.Caption := sl.Strings[i];

      subMenu.Add(menuItem);
      subMenu := menuItem;

    end;

  finally
    sl.Free;
  end;

  Result := menuItem;
end;

function TForm1.ValidName(Name: string): string;
var
  i: integer;
begin
  for i:=1 to Length(Name) do
    if (Name[i] = '&')
    or (Name[i] = '.')
    or (Name[i] = ' ')
    or (Name[i] = '-') then
      Name[i] := '_';

  Result := Name;
end;


function TForm1.FindSubMenu( Name: string ): TMenuItem;
var
  i: integer;
begin
  Result := nil;
  for i:=0 to MainMenu.Items.Count-1 do
  begin
    if MainMenu.Items[i].Name = Name then
      Result := MainMenu.Items[i]
    else
      Result := FindSubMenuItem(MainMenu.Items[i],Name);

    if Result <> nil then
      exit;
  end;
end;

function TForm1.FindSubMenuItem(SubMenu: TMenuItem;
  Name: string): TMenuItem;
var
  i: integer;
begin
  Result := nil;
  if (SubMenu = nil) or (Name = '') then
    exit;

  for i:=0 to SubMenu.Count-1 do
  begin
    if SubMenu.Items[i].Name = Name then
    begin
      Result := SubMenu.Items[i];
      exit;
    end else
      Result := FindSubMenuItem(SubMenu.Items[i],Name);

    if Result <> nil then
      exit;
  end;
end;

procedure TForm1.RemoveDataLinks(FromItem: TMenuItem; AppObject: TAppObject);
var
  i: integer;
  item: TMenuItem;
begin
  if AppObject <> nil then
  begin
    for i:=FromItem.Count-1 downto 0 do
    begin
      item := FromItem.Items[i];
      // Search recursive...
      if item.Count > 0 then
        RemoveDataLinks(Item,AppObject);

      if Pointer(Item.Tag) = AppObject then
      begin
        FromItem.Remove(Item);
        Item.Free;
      end;
    end;
  end;
end;

procedure TForm1.RemoveEmptyLinks(FromItem: TMenuItem; AppObject: TAppObject);
var
  i: integer;
  item: TMenuItem;
begin
  if AppObject <> nil then
  begin
    for i:=FromItem.Count-1 downto 0 do
    begin
      item := FromItem.Items[i];
      // Search recursive...
      if item.Count > 0 then
        RemoveEmptyLinks(Item,AppObject);

      if not Assigned(Item.OnClick)
      and (Item.Count = 0) then
      begin
        FromItem.Remove(item);
        Item.Free;
      end;
    end;
  end;
end;


procedure TForm1.AddMenuItem(AppObject: TAppObject);
var
  menuItem, submenu: TMenuItem;
begin
  submenu := GetOrCreateSubMenuItem(AppObject.MenuPath);
  if submenu <> nil then
  begin
    menuItem := TMenuItem.Create(self);
    menuItem.Caption := AppObject.Caption;
    menuItem.OnClick := OnMenuClick;
    menuItem.Tag := integer(AppObject);
    if Assigned(AppObject.Glyph) then
    begin
      menuItem.Bitmap := AppObject.Glyph;
    end else
      if Assigned(AppObject.ImageList)
      and (AppObject.ImageIndex >= 0) then
      begin
        menuItem.Bitmap := TBitmap.Create;
        AppObject.ImageList.GetBitmap(AppObject.ImageIndex,menuItem.Bitmap)
      end;

    submenu.Add(menuItem);
  end;
end;

procedure TForm1.OnMenuClick(Sender: TObject);
var
  cmdObj: TAppObject;
begin
  cmdObj := TAppObject( TMenuItem(Sender).Tag );
  if Assigned(cmdObj) then
    cmdObj.Execute;
end;


procedure TForm1.FormDestroy(Sender: TObject);
begin
  // Free Main Menu if we get unloaded...
  // If it's the end of the application,
  // the MainForm will do this.
  if Assigned(MainMenu)
  and not _Closing then
    FreeAndNil(MainMenu);
end;

initialization
  Form1 := TForm1.Create(nil);

finalization
  Form1.Free;


end.
