unit TreeManagerUnit;

interface

uses
  inifiles, strutils, menus, extctrls,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BPLBase, BPLClient, ImgList, ComCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    BPLTreeManager: TDynBPLClient;
    ImageList: TImageList;
    Splitter: TSplitter;
    tv: TTreeView;
    PopupMenu1: TPopupMenu;
    Left1: TMenuItem;
    Right1: TMenuItem;
    op1: TMenuItem;
    Bottom1: TMenuItem;
    N1: TMenuItem;
    Expand1: TMenuItem;
    Collapseall1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure BPLTreeManagerAppObjectNotification(Sender: TObject;
      AppObject: TAppObject; Notification: TAppObjectNotification);
    procedure BPLTreeManagerServiceCall(Sender: TObject;
      ServiceText: String; ServiceNumber: Integer; Subject: TObject;
      out handled: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PopupClick(Sender: TObject);
    procedure tvDblClick(Sender: TObject);
  private
    _Closing: boolean;
    IniFileName: string;

    function GetOrCreateNode(AppObject: TAppObject): TTreeNode;
    function FindNode(Root: TTreeNode; Path: string): TTreeNode;
    procedure RemoveEmptyLinks(AppObject: TAppObject);
    procedure RemoveObjectLinks(AppObject: TAppObject);
    procedure AddTreeItem(AppObject: TAppObject);
    procedure ChangeAlignment( Align: integer );

    procedure LoadFromIniFile;
    procedure SaveToIniFile;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation


{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
  appObj: TAppObject;
begin
  _Closing := false;

  appObj :=
    BPLTreeManager.GetNextAppObject(nil, otPath, 'AppConfigFile');
  if appObj <> nil then
    IniFileName := appObj.Text;
  if IniFileName = '' then
    IniFileName := ChangeFileExt( Application.ExeName, '.ini' );

  Splitter.Parent := Application.MainForm;
  tv.Parent := Application.MainForm;

  LoadFromIniFile;
  BPLTreeManager.IterateAppObjects;
end;

procedure TForm1.BPLTreeManagerAppObjectNotification(Sender: TObject;
  AppObject: TAppObject; Notification: TAppObjectNotification);
begin
  if _Closing then
    exit;

  case Notification of
    noRegister,
    noIterate:
      begin
        case AppObject.ObjectType of
          otConfig,
          otCommand:
            begin
              AddTreeItem(AppObject);
            end;
        end;
      end;
    noUnregister,
    noComponentUnassigning:
      begin
        case AppObject.ObjectType of
          otConfig,
          otCommand:
            begin
              RemoveObjectLinks(AppObject);
              RemoveEmptyLinks(AppObject);
            end;
        end;
      end;
  end;

end;


procedure TForm1.BPLTreeManagerServiceCall(Sender: TObject;
  ServiceText: String; ServiceNumber: Integer; Subject: TObject;
  out handled: Boolean);
begin
  if ServiceNumber = 1 then // App.End
    _Closing := true;
end;

procedure TForm1.RemoveObjectLinks(AppObject: TAppObject);
var
  i: integer;
  item: TTreeNode;
begin
  if AppObject <> nil then
  begin
    for i:=tv.Items.Count-1 downto 0 do
    begin
      item := tv.Items[i];
      if Pointer(Item.Data) = AppObject then
        tv.Items.Delete(Item);
    end;
  end;
end;

procedure TForm1.RemoveEmptyLinks(AppObject: TAppObject);
var
  i: integer;
  item: TTreeNode;
begin
  if AppObject <> nil then
  begin
    for i:=tv.Items.Count-1 downto 0 do
    begin
      item := tv.Items[i];
      if (item.Data = nil)
      and not item.HasChildren then
        tv.Items.Delete(Item);
    end;
  end;
end;


procedure TForm1.AddTreeItem(AppObject: TAppObject);
var
  node, parent: TTreeNode;
begin
  parent := GetOrCreateNode(AppObject);
  if parent <> nil then
  begin
    node := tv.Items.AddChild(parent,AppObject.Caption);
    node.Data := AppObject;
    node.ImageIndex := 1;

    if Assigned(AppObject.Glyph) then
    begin
      node.ImageIndex :=
        ImageList.AddMasked(AppObject.Glyph,AppObject.Glyph.TransparentColor);
    end else
    begin
      if Assigned(AppObject.ImageList)
      and (AppObject.ImageIndex >= 0) then
        node.ImageIndex :=
          ImageList.AddImage(AppObject.ImageList,AppObject.ImageIndex);
    end;
    node.SelectedIndex := node.ImageIndex;
  end;
end;


procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  tv.Parent := self;
  Splitter.Parent := self;
end;

function TForm1.GetOrCreateNode(AppObject: TAppObject): TTreeNode;
var
  i: integer;
  sl: TStringList;
begin
  Result := FindNode(nil,AppObject.MenuPath);
  if Result <> nil then
    exit;

  sl := TStringList.Create;
  try
    sl.Delimiter := '.';
    sl.DelimitedText := AppObject.MenuPath;

    for i:=0 to sl.Count-1 do
    begin
      Result := tv.Items.AddChild(Result,sl.Strings[i]);
      Result.Text := sl.Strings[i];
    end;

  finally
    sl.Free;
  end;

end;


function TForm1.FindNode(Root: TTreeNode; Path: string): TTreeNode;
var
  i: integer;
  node: TTreeNode;
  caption: string;
begin
  Result := nil;
  i := Pos('.',Path);
  if i = 0 then
    caption := Path
  else
    Caption := LeftStr(Path,i-1);

  if Root = nil then
    node := tv.Items.GetFirstNode
  else
    node := Root.getFirstChild;

  while node <> nil do
  begin
    if node.Text = Caption then
    begin
      if i > 0 then
        Result := FindNode(node,RightStr(Path,Length(Path)-i))
      else
        Result := node;
        
      if Result <> nil then
        exit;
    end else
      node := Root.GetNextChild(node);
  end;
end;

procedure TForm1.PopupClick(Sender: TObject);
begin
  case TMenuItem(Sender).Tag of
    1,2,3,4:
      begin
        ChangeAlignment(TMenuItem(Sender).Tag);
        SaveToIniFile;
      end;
    5: tv.FullExpand;
    6: tv.FullCollapse;
  end;
end;

procedure TForm1.tvDblClick(Sender: TObject);
begin
  if tv.Selected <> nil then
    if tv.Selected.Data <> nil then
      TAppObject(tv.Selected.Data).Execute;
end;

procedure TForm1.ChangeAlignment(Align: integer);
begin
  case Align of
    1: begin
         // Because of creation order:
         if tv.Align in [alTop, alBottom] then
         begin
           Splitter.Align := alLeft;
           tv.Align := alLeft;
         end else
         begin
           tv.Align := alLeft;
           Splitter.Align := alLeft;
         end;
       end;
    2: begin
         tv.Align := alRight;
         Splitter.Align := alRight;
       end;
    3: begin
         tv.Align := alTop;
         Splitter.Align := alTop;
       end;
    4: begin
         tv.Align := alBottom;
         Splitter.Align := alBottom;
       end;
  end;
end;

procedure TForm1.LoadFromIniFile;
var
  ini: TIniFile;
begin
  ini := TIniFile.Create(IniFileName);
  try
    ChangeAlignment(ini.ReadInteger('TreeManager','Align',1));
  finally
    ini.Free;
  end;
end;

procedure TForm1.SaveToIniFile;
var
  ini: TIniFile;
  al: integer;
begin
  ini := TIniFile.Create(IniFileName);
  try
    al := 1;
    case tv.Align of
      alRight:    al := 2;
      alTop:      al := 3;
      alBottom:   al := 4;
    end;
    ini.WriteInteger('TreeManager','Align',al);
  finally
    ini.Free;
  end;
end;

initialization
  Form1 := TForm1.Create(nil);

finalization
  Form1.Free;


end.
